@@ -54,7 +54,7 @@ require 'config/opts'
 require 'config/config_os_arch'
 require 'config/config_fortran'
 
-include Config
+include JblasConfig
 
 ATLAS_REQUIRED_SYMBOLS = [
   'dsyev_', # eigenvalue function not yet included in ATLAS/LAPACK
@@ -65,7 +65,7 @@ ATLAS_REQUIRED_SYMBOLS = [
   'ATL_caxpy'
 ]
 
-LAPACK_REQUIRED_SYMBOLS = [ 'dsyev_', 'daxpy_' ]
+LAPACK_REQUIRED_SYMBOLS = [ 'dsyev_', 'daxpy_', 'dgemm_' ]
 
 ATLAS_LIBS = %w(lapack lapack_fortran lapack_atlas f77blas cblas atlas)
 PT_ATLAS_LIBS = %w(lapack lapack_fortran lapack_atlas ptf77blas ptcblas atlas)
@@ -113,17 +113,22 @@ end
 desc 'looking for libraries...'
 configure 'LOADLIBES' => ['LINKAGE_TYPE', :libpath, 'F77', 'BUILD_TYPE', 'OS_ARCH'] do
 
-  case CONFIG['BUILD_TYPE']
-  when 'atlas'
-    if $opts.defined? :ptatlas
-      libs = PT_ATLAS_LIBS
-    else
-      libs = ATLAS_LIBS
-    end
-    syms = ATLAS_REQUIRED_SYMBOLS
-  when 'lapack'
-    libs = LAPACK_LIBS
+  if $opts.defined? :libs
+    libs = $opts[:libs].split(',')
     syms = LAPACK_REQUIRED_SYMBOLS
+  else
+    case CONFIG['BUILD_TYPE']
+    when 'atlas'
+      if $opts.defined? :ptatlas
+        libs = PT_ATLAS_LIBS
+      else
+        libs = ATLAS_LIBS
+      end
+      syms = ATLAS_REQUIRED_SYMBOLS
+    when 'lapack'
+      libs = LAPACK_LIBS
+      syms = LAPACK_REQUIRED_SYMBOLS
+    end
   end
 
   result = LibHelpers.find_libs(CONFIG[:libpath], libs, syms)
@@ -142,28 +147,33 @@ configure 'LOADLIBES' => ['LINKAGE_TYPE', :libpath, 'F77', 'BUILD_TYPE', 'OS_ARC
     CONFIG['LOADLIBES'] += result.keys.
       sort {|x, y| libs.index(x) <=> libs.index(y)}.
       map {|s| File.join(result[s], LibHelpers.libname(s)).escape }
-    if CONFIG['F77'] == 'gfortran'
+    if CONFIG['F77'] =~ /gfortran$/
       puts CONFIG['OS_ARCH']
       if CONFIG['OS_NAME'] == 'Linux' and CONFIG['OS_ARCH'] == 'amd64'
-	CONFIG['LOADLIBES'] += ['-lgfortran']
+	      CONFIG['LOADLIBES'] += ['-lgfortran']
         puts <<EOS
 WARNING: on 64bit Linux, I cannot link the gfortran library into the shared library
 because it's usually not compiled with -fPIC. This means that you need to
 have libgfortran.so installed on your target system. Sorry for the inconvenience!
 EOS
+      elsif CONFIG['OS_NAME'] == 'Mac\ OS\ X'
+        print "Looking for where libgfortran.a is... "
+        libgfortran_path = %x(gfortran -print-file-name=libgfortran.a).strip
+	      puts "(#{libgfortran_path})"
+	      CONFIG['LOADLIBES'] += [libgfortran_path]
       else
         CONFIG['LOADLIBES'] += ['-l:libgfortran.a']
       end
     end
-    if CONFIG['OS_NAME'] == 'Mac\ OS\ X'
-      CONFIG['LOADLIBES'] += ['/opt/local/lib/gcc43/libgfortran.a']
-    end 
+    #if CONFIG['OS_NAME'] == 'Mac\ OS\ X'
+    #  CONFIG['LOADLIBES'] += ['/opt/local/lib/gcc43/libgfortran.a']
+    #end 
   end
   ok
 end
 
 if __FILE__ == $0
   $opts = Opts.new(ARGV)
-  Config.run :libs
-  Config::CONFIG.dump($stdout)
+  JblasConfig.run :libs
+  JblasConfig::CONFIG.dump($stdout)
 end
