@@ -32,6 +32,8 @@
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ## --- END LICENSE BLOCK ---
 
+$: << "."
+
 require 'config/path'
 require 'config/config'
 require 'config/opts'
@@ -44,7 +46,7 @@ require 'config/config_make'
 require 'config/config_lapack_sources'
 require 'config/config_libs'
 
-include Config
+include JblasConfig
 include Path
 
 args = []
@@ -82,6 +84,7 @@ options summary:
                            (default for Windows!)
   --ptatlas                Link against multithreaded versions of ATLAS libraries
   --arch-flavor=...        Set architectural flavor (e.g. --arch-flavor=sse2)
+  --libs=lib1,lib2,...     Override libraries to search in
 EOS
 
 configure :all => [:os_arch, :tools, :java, :cc, :fortran, :make, :lapack_sources, :libs]
