@@ -37,10 +37,10 @@
 
 package org.jblas;
 
-import org.jblas.exceptions.LapackException;
-import org.jblas.exceptions.LapackArgumentException;
-import org.jblas.exceptions.LapackConvergenceException;
-import org.jblas.exceptions.LapackSingularityException;
+import org.jblas.exceptions.*;
+import org.jblas.util.Functions;
+
+import static org.jblas.util.Functions.*;
 
 //import edu.ida.core.OutputValue;
 
@@ -161,8 +161,14 @@ public class SimpleBlas {
         return NativeBlas.idamax(x.length, x.data, 0, 1) - 1;
     }
 
+  /**
+   * Compute index of element with largest absolute value (complex version).
+   *
+   * @param x matrix
+   * @return index of element with largest absolute value.
+   */
     public static int iamax(ComplexDoubleMatrix x) {
-        return NativeBlas.izamax(x.length, x.data, 0, 1);
+        return NativeBlas.izamax(x.length, x.data, 0, 1) - 1;
     }
 
     /***************************************************************************
@@ -393,6 +399,63 @@ public class SimpleBlas {
         }
     }
 
+  /**
+   * Generalized Least Squares via *GELSD.
+   *
+   * Note that B must be padded to contain the solution matrix. This occurs when A has fewer rows
+   * than columns.
+   *
+   * For example: in A * X = B, A is (m,n), X is (n,k) and B is (m,k). Now if m < n, since B is overwritten to contain
+   * the solution (in classical LAPACK style), B needs to be padded to be an (n,k) matrix.
+   *
+   * Likewise, if m > n, the solution consists only of the first n rows of B.
+   *
+   * @param A an (m,n) matrix
+   * @param B an (max(m,n), k) matrix (well, at least)
+   */
+    public static void gelsd(DoubleMatrix A, DoubleMatrix B) {
+      int m = A.rows;
+      int n = A.columns;
+      int nrhs = B.columns;
+      int minmn = min(m, n);
+      int maxmn = max(m, n);
+
+      if (B.rows < maxmn) {
+        throw new SizeException("Result matrix B must be padded to contain the solution matrix X!");
+      }
+
+      int smlsiz = NativeBlas.ilaenv(9, "DGELSD", "", m, n, nrhs, 0);
+      int nlvl = max(0, (int) log2(minmn/ (smlsiz+1)) + 1);
+
+//      System.err.printf("GELSD\n");
+//      System.err.printf("m = %d, n = %d, nrhs = %d\n", m, n, nrhs);
+//      System.err.printf("smlsiz = %d, nlvl = %d\n", smlsiz, nlvl);
+//      System.err.printf("iwork size = %d\n", 3 * minmn * nlvl + 11 * minmn);
+
+      int[] iwork = new int[3 * minmn * nlvl + 11 * minmn];
+      double[] s = new double[minmn];
+      int[] rank = new int[1];
+      int info = NativeBlas.dgelsd(m, n, nrhs, A.data, 0, m, B.data, 0, B.rows, s, 0, -1, rank, 0, iwork, 0);
+      if (info == 0) {
+        return;
+      } else if (info < 0) {
+        throw new LapackArgumentException("DGESD", -info);
+      } else if (info > 0) {
+        throw new LapackConvergenceException("DGESD", info + " off-diagonal elements of an intermediat bidiagonal form did not converge to 0.");
+      }
+    }
+
+  public static void geqrf(DoubleMatrix A, DoubleMatrix tau) {
+    int info = NativeBlas.dgeqrf(A.rows, A.columns, A.data, 0, A.rows, tau.data, 0);
+    checkInfo("GEQRF", info);
+  }
+
+  public static void ormqr(char side, char trans, DoubleMatrix A, DoubleMatrix tau, DoubleMatrix C) {
+    int k = tau.length;
+    int info = NativeBlas.dormqr(side, trans, C.rows, C.columns, k, A.data, 0, A.rows, tau.data, 0, C.data, 0, C.rows);
+    checkInfo("ORMQR", info);
+  }
+
 //BEGIN
   // The code below has been automatically generated.
   // DO NOT EDIT!
@@ -503,8 +566,14 @@ public class SimpleBlas {
         return NativeBlas.isamax(x.length, x.data, 0, 1) - 1;
     }
 
+  /**
+   * Compute index of element with largest absolute value (complex version).
+   *
+   * @param x matrix
+   * @return index of element with largest absolute value.
+   */
     public static int iamax(ComplexFloatMatrix x) {
-        return NativeBlas.icamax(x.length, x.data, 0, 1);
+        return NativeBlas.icamax(x.length, x.data, 0, 1) - 1;
     }
 
     /***************************************************************************
@@ -728,5 +797,62 @@ public class SimpleBlas {
         }
     }
 
+  /**
+   * Generalized Least Squares via *GELSD.
+   *
+   * Note that B must be padded to contain the solution matrix. This occurs when A has fewer rows
+   * than columns.
+   *
+   * For example: in A * X = B, A is (m,n), X is (n,k) and B is (m,k). Now if m < n, since B is overwritten to contain
+   * the solution (in classical LAPACK style), B needs to be padded to be an (n,k) matrix.
+   *
+   * Likewise, if m > n, the solution consists only of the first n rows of B.
+   *
+   * @param A an (m,n) matrix
+   * @param B an (max(m,n), k) matrix (well, at least)
+   */
+    public static void gelsd(FloatMatrix A, FloatMatrix B) {
+      int m = A.rows;
+      int n = A.columns;
+      int nrhs = B.columns;
+      int minmn = min(m, n);
+      int maxmn = max(m, n);
+
+      if (B.rows < maxmn) {
+        throw new SizeException("Result matrix B must be padded to contain the solution matrix X!");
+      }
+
+      int smlsiz = NativeBlas.ilaenv(9, "DGELSD", "", m, n, nrhs, 0);
+      int nlvl = max(0, (int) log2(minmn/ (smlsiz+1)) + 1);
+
+//      System.err.printf("GELSD\n");
+//      System.err.printf("m = %d, n = %d, nrhs = %d\n", m, n, nrhs);
+//      System.err.printf("smlsiz = %d, nlvl = %d\n", smlsiz, nlvl);
+//      System.err.printf("iwork size = %d\n", 3 * minmn * nlvl + 11 * minmn);
+
+      int[] iwork = new int[3 * minmn * nlvl + 11 * minmn];
+      float[] s = new float[minmn];
+      int[] rank = new int[1];
+      int info = NativeBlas.sgelsd(m, n, nrhs, A.data, 0, m, B.data, 0, B.rows, s, 0, -1, rank, 0, iwork, 0);
+      if (info == 0) {
+        return;
+      } else if (info < 0) {
+        throw new LapackArgumentException("DGESD", -info);
+      } else if (info > 0) {
+        throw new LapackConvergenceException("DGESD", info + " off-diagonal elements of an intermediat bidiagonal form did not converge to 0.");
+      }
+    }
+
+  public static void geqrf(FloatMatrix A, FloatMatrix tau) {
+    int info = NativeBlas.sgeqrf(A.rows, A.columns, A.data, 0, A.rows, tau.data, 0);
+    checkInfo("GEQRF", info);
+  }
+
+  public static void ormqr(char side, char trans, FloatMatrix A, FloatMatrix tau, FloatMatrix C) {
+    int k = tau.length;
+    int info = NativeBlas.sormqr(side, trans, C.rows, C.columns, k, A.data, 0, A.rows, tau.data, 0, C.data, 0, C.rows);
+    checkInfo("ORMQR", info);
+  }
+
 //END
 }
